/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2005 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <gtkmm.h>

#include "libtkcommon/exceptions.h"
#include "libtkdb/database.h"
#include "libtkresolver/sdl.h"
#include "catcher.h"

using namespace TkResolver;
using namespace Experimenter;


struct ExceptionDetails
{
  bool          occurred;
  Glib::ustring type;
  Glib::ustring message;

  ExceptionDetails(const Glib::ustring & typ, const Glib::ustring & msg)
    : occurred(true), type(typ), message(msg) {}

  ExceptionDetails()
    : occurred(false) {}
};

bool
TkResolver::Experimenter:: catch_exception(Generator &gen, bool interactive)
{
  ExceptionDetails exception_details;

  try {
    gen();
  }
  catch (const UnixError &e)
    {
      exception_details = ExceptionDetails("System error", e.what());
    }
  catch (const std::domain_error &e)
    {
      exception_details = ExceptionDetails("Domain error", e.what());
    }
  catch (const std::invalid_argument &e)
    {
      exception_details = ExceptionDetails("Invalid argument", e.what());
    }
  catch (const std::underflow_error &e)
    {
      exception_details = ExceptionDetails("Underflow error", e.what());
    }
  catch (const std::runtime_error &e)
    {
      exception_details = ExceptionDetails("Runtime error", e.what());
    }
  catch (const std::exception & e)
    {
      exception_details = ExceptionDetails("Error", e.what());
    }
  catch (...)
    {
      exception_details = ExceptionDetails("Unknown error", "Unknown error");
    }
  
  SDL::reset();
  
  if (interactive && exception_details.occurred)
    {
      Gtk::MessageDialog dialog(exception_details.message, false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
      dialog.set_title(exception_details.type);
      dialog.run();
    }
  return exception_details.occurred;
}
