/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2005-2008 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include "config.h"

#include <wchar.h>

#include <stdlib.h>
#include <getopt.h>

#include <gtkmm.h>
#include <gconfmm.h>

#include "libtkcommon/debug.h"
#include "libtkdbui/login.h"
#include "experimenter.h"

using namespace TkResolver;
using namespace Experimenter;

#define _(x) x

void
print_help ()
{
  std::cout << PACKAGE_NAME << " version " << PACKAGE_VERSION << std::endl;
  std::wcout << L"Copyright (C) 2005 Robert Ham (rah@bash.sh)" << std::endl;
  std::wcout << std::endl;
  std::wcout << L"This program comes with ABSOLUTELY NO WARRANTY.  You are licensed to use it" << std::endl;
  std::wcout << L"under the terms of the GNU General Public License, version 2 or later.  See" << std::endl;
  std::wcout << L"the COPYING file that came with this software for details." << std::endl;
  std::wcout << std::endl;
  std::wcout << L" -h, --help                  " << _("Display this help info") << std::endl;
  std::wcout << std::endl;
}

struct ExceptionDetails
{
  bool          occurred;
  Glib::ustring type;
  Glib::ustring message;

  ExceptionDetails(const Glib::ustring & typ, const Glib::ustring & msg)
    : occurred(true), type(typ), message(msg) {}

  ExceptionDetails()
    : occurred(false) {}
};

int
main (int argc, char ** argv)
{

  int opt;
  const char * options = "h";
  struct option long_options[] = {
    { "help",        no_argument,       NULL, 'h' },
    { 0, 0, 0, 0 }
  };
  while ((opt = getopt_long (argc, argv, options, long_options, NULL)) != -1) {
    switch (opt) {
      
    case 'h':
      print_help ();
      exit (0);
      break;
    case ':':
    case '?':
      print_help ();
      exit (1);
      break;
    }
  }

  Gtk::Main kit(argc, argv);
  //Gnome::Main::Main("TkResolver++","1.0", Gnome::module_info_get(), argc, argv);
  Gnome::Conf::init();

  ExceptionDetails exception_details;

  try {
    DEBUG_OUT << "boo" << std::endl;

    Db::Ui::Login * login( new Db::Ui::Login() );
    int login_response = login->run();
    if (login_response != Gtk::RESPONSE_OK)
      {
	delete login;
	exit(0);
      }

    String data_source_name(login->get_data_source_name());
    String username(login->get_username());
    String password(login->get_password());
    delete login;
    
    if (data_source_name.size() < 1)
      {
	exit(0);
      }

    DEBUG_OUT << "Creating Experimenter" << std::endl;
    TkResolver::Experimenter::Experimenter experimenter(data_source_name,
							username,
							password);
    DEBUG_OUT << "Experimenter created; running" << std::endl;
    
    Gtk::Main::run(experimenter);
  }
  catch (const std::invalid_argument & ia)
    {
      exception_details = ExceptionDetails("Invalid argument", ia.what());
    }
  catch (const std::underflow_error & ue)
    {
      exception_details = ExceptionDetails("Underflow error", ue.what());
    }
  catch (const std::runtime_error & re)
    {
      exception_details = ExceptionDetails("Runtime error", re.what());
    }
  catch (const std::exception & except)
    {
      exception_details = ExceptionDetails("Error", except.what());
    }
  catch (...)
    {
      exception_details = ExceptionDetails("Unknown error", "Unknown error");
    }

  if (exception_details.occurred)
    {
      std::wcerr << exception_details.type << L": " << exception_details.message << std::endl;
      
      Gtk::MessageDialog *dialog( new Gtk::MessageDialog(exception_details.message, false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true) );
      dialog->set_title(exception_details.type);
      dialog->show_all_children();
      dialog->show();
      dialog->run();
      
      exit(1);
    }
  
  exit(0);
}
