/* -*- c++ -*-
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004-2007 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_DATABASE_H__
#define __TK_DATABASE_H__

#define GETTEXT_PACKAGE "TkResolver"

#include <stdexcept>

#include <odbc++/preparedstatement.h>

#include "libtkdb/database_types.h"
#include "libtkcommon/tkstring.h"
#include "libtkrandom/random_types.h"
#include "libtkresolver/types.h"

#define EXPERIMENT_GROUP "Test group"

namespace TkResolver {
  namespace Db {
    
    TkResolver::String date_time_string (time_t t);
    
    typedef std::pair<Code, String> IntentInfo;
    typedef std::list<IntentInfo> IntentInfoList;
    
    class Database
      {
      private:
	typedef std::map<String, Type> FieldInfo;
	typedef std::map<String, FieldInfo> TableInfo;

	odbc::Connection    * _connection;

	TableInfo           _table_info;
	
	void get_table_info(const String &name);
	void check_table_info(const String &name);
	
	Id insert_property(const Id &property_type_id);
	Id insert_property_value (const Id &property_id);
	void insert_property_value_data (const Id &property_value_id, const String &type_name, const String &table_name, const void *value);
	void insert_experiment_property(const Id &experiment_id, const Id &property_id);
	Id experiment_set_property(const Id &experiment_id, const String &property_name, const void *value);
	
	
	Id insert_experiment(const Id &location_id, const Random::Bits &randomness);
	
	void experiment_insert_operators(const Id &experiment_id, const IdList &operators);
	void experiment_insert_result(const Id &experiment_id, const Result &result);
	
      public:
	Database (const TkResolver::String &data_source_name = TkResolver::String("TkResolver++"),
		  const TkResolver::String &username = TkResolver::String("psi"),
		  const TkResolver::String &password = TkResolver::String("isp"));
	~Database();

	Statement  * new_statement();
	void         execute_statement(const TkResolver::String &sql);
	DataSet    * execute_query(const TkResolver::String &sql);
	PStatement * prepare_statement(const TkResolver::String &sql);
	
	Type         get_column_type(const String &table, const String &column);
	DataSet *    get_table_data(const String &table, const StringList &columns);
	PStatement * get_table_insert_statement(const String &table, const StringList &columns);
	Id           get_last_insert_id();
	
	void property_type_get_details (const String &property_name,
					Id *property_type_id,
					String *property_value_type_name,
					String *property_value_table_name);
	
	Id           lookup_location(const String &location_name);
	Id           choose_location();
	Id           choose_operator();
	Code         choose_intent(unsigned trace_count);
	IntentInfoList get_intent_options(unsigned trace_count);
	unsigned     get_experiment_screen_width(const Id &experiment_id);
	unsigned     get_experiment_trace_count(const Id &experiment_id);
	String       get_experiment_algorithm(const Id &experiment_id);
	Code         get_experiment_intent_code(const Id &experiment_id);
	DataSet *    get_results(const Id &experiment);
	
	Id insert_experiment(const Id &location_id,
			     const Random::Bits &randomness,
			     
			     const DateTime &time_stamp,
			     const bool test,
			     const StringValue &algorithm_spec,
			     const Unsigned &bit_multiplier,
			     const Unsigned &trace_count,
			     const Unsigned &time,
			     const Bool &scrolled,
			     const Double &speed,
			     const Double &horizon,
			     const StringValue &notes,
			     const StringValue &intent_code,
			     const Double &ability_rating,
			     const Unsigned &width,
			     const Unsigned &height,
			     const TkResolver::String &retriever,
			     const Bool &control,
			     
			     const IdList &operators,
			     const Result &result);
			     };

  }; /* namespace Db */
}; /* namespace TkResolver */

#endif /* __TK_DATABASE_H__ */
