/* -*- c++ -*-
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Should you need to contact me, the author, you can do so by
 * e-mail - mail your message to Viktor T. Toth at <vttoth@vttoth.com>.
 *
 * Updated 2007 by Robert Ham for tkresolver
 *
 * Copyright (C) 2007 Robert Ham <rah@bash.sh> for tkresolver
 * Copyright (C) Viktor T. Toth at <vttoth@vttoth.com>
 *
 */

#ifndef __TK_MP_MATRIX_H__
#define __TK_MP_MATRIX_H__

#include <iostream>
#include <complex>
#include <stdexcept>

#include <gmpxx.h>

#include "real.h"
#include "integer.h"

namespace TkResolver {
  namespace MP {
    
    template <typename T>
    class Matrix
    {
      class Row
      {
      private:
	T *_data;
      public:
	Row(const Integer::Number &n);
	~Row();
	T &operator[](const Integer::Number &c);
	const T &operator[](const Integer::Number &c) const;
      };
  
    private:
      typedef Row* RowPtr;
  
      RowPtr         *_rdata;
      Integer::Number _nrows;
      Integer::Number _ncols;
  
    public:
      Matrix (const Integer::Number &r, const Integer::Number &c);
      Matrix(const Matrix &op);
      ~Matrix();
      Matrix &operator=(const Matrix &op);
      operator Matrix< std::complex<T> > () const;
      operator Matrix< Real::Number > () const;
  
      Row &operator[](const Integer::Number &r);
      const Row &operator[](const Integer::Number &r) const;
      Matrix sub(const Integer::Number &r, const Integer::Number &c) const;

      Matrix &operator*=(const T &op);
      Matrix operator*(const T &op) const;
      Matrix &operator/=(const T &op);
      Matrix operator/(const T &op) const;
      Matrix operator*(const Matrix &op) const;
  
    };

    template<typename T> std::ostream &operator<<(std::ostream &os, Matrix<T> const &op);
    
#include "matrix_impl.h"

  } /* namespace MP */
} /* namespace TkResolver */


#endif /* __TK_MP_MATRIX_H__ */
