/* -*- c++ -*-
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2008 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_QUERY_TABLES_H__
#define __TK_QUERY_TABLES_H__

#include "libtkcommon/tkstring.h"
#include "libtkdb/database.h"

namespace TkResolver {
  namespace Query {
    
    typedef String TableName;
    typedef String TableAlias;
    
    class QueryTables
      {
      private:
	typedef unsigned TableId;
	typedef std::map<TableAlias, TableName> TableMap;
	
	TableMap     _tables;
	TableId      _next_table_no;
	
	StringList   _link_clauses;
	
	Db::Database    *_database;
	
      public:
	QueryTables(Db::Database *database);
	
	TableAlias add_table(const TableName &name);
	TableAlias add_property_value_table(const String &identifier);
	void add_link_clause(const String &clause);
	
	TkResolver::String get_sql() const;
      };
    
  }; /* namespace Query */
}; /* namespace TkResolver */

#endif /* __TK_QUERY_TABLES_H__ */
