/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "libtkcommon/debug.h"
#include "pseudo_random.h"
#include "retriever_registry.h"

using namespace TkResolver;
using namespace Random;


/*
from random(3):
       Current  "optimal"  values for the size of the state array n are 8, 32,
       64, 128, and 256 bytes; other amounts will be rounded down to the near-
       est known amount.  Using less than 8 bytes will cause an error.
*/
char PseudoRandom::_state[256];

bool PseudoRandom::_state_initialised(false);


PseudoRandom:: PseudoRandom(unsigned chars)
  : Retriever(chars)
{
  initialise_state();
}

void
PseudoRandom:: initialise_state()
{
  if (_state_initialised)
    return;

  /* _old_state = */
  initstate (time(NULL), _state, 256);

  _state_initialised = true;
}

PseudoRandom:: ~PseudoRandom()
{
  0;
  //  setstate (_old_state);
}

Buffer *
PseudoRandom:: retrieve()
{
  Buffer * buf;
  char * old_state;
  char data;
  
  buf = new Buffer;

  for (unsigned i = 0; i < get_count(); i++)
    {
      data = 0;
      for (unsigned short j = 0; j < BITS_PER_CHAR; j++)
        {
	  if (random() > (RAND_MAX / 2))
	    data |=  (1 << j);
	}

      buf->push_back (data);
    }

  return buf;
}

Retriever *
PseudoRandomType:: new_retriever(unsigned count)
{
  return new PseudoRandom(count);
}

unsigned
PseudoRandomType:: get_preferred_block_size()
{
  return 256;
}

bool
PseudoRandomType:: is_true_randomness()
{
  return false;
}
