/* -*- c++ -*-
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004-2007 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#ifndef __TK_TKRESOLVER_H__
#define __TK_TKRESOLVER_H__

#include <list>
#include <utility>

#include "libtkcommon/tkstring.h"
#include "libtkcommon/tkvalue.h"
#include "libtkalgorithm/algorithm.h"
#include "libtkrandom/decision_tree.h"
#include "libtkrandom/randomness.h"
#include "sdl.h"
#include "display.h"
#include "types.h"

namespace TkResolver {

  class TKResolver
  {
  public:
    typedef Uint32 tk_time_t;

    static const short MILLISECONDS_PER_SECOND = 1000;

  private:
    unsigned             _trace_count;
    unsigned             _total_time; /* seconds */

    SDL                  _sdl;
    Display            * _display;
    Algorithm            _algorithm;
    Random::Randomness   _rand;
    Random::DecisionTree _decision_tree;
    bool                 _test;

    Line                 _current_line;
    std::list<Line>      _offsets;
  
  
    /** returns true on quit command */
    bool control();
    void slice();
    Result get_result();
    
  public:
    TKResolver(const String &retriever_type,
	       const String &algo,
	       unsigned bit_multiplier,
	       unsigned trace_count,
	       unsigned total_time,
	       unsigned width,
	       unsigned height,
	       bool scrolled = false,
	       double speed = 0.0,
	       double horizon = 0.0,
	       bool test = true);
    ~TKResolver();

    unsigned get_display_width() const;
    unsigned get_display_height() const;
    unsigned get_total_lines() const;
    const Random::Bits & get_used_bits() const;
    const Result & run();
  };

}; /* namespace TkResolver */

#endif /* __TK_TKRESOLVER_H__ */
