/* 
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2007 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <memory>
#include <iostream>

#include <odbc++/drivermanager.h>
#include <odbc++/databasemetadata.h>
#include <odbc++/resultset.h>

#include "libtkcommon/debug.h"
#include "libtkcommon/tkstring.h"

int
main(int argc, char **argv)
{
  using namespace TkResolver;
  String dsn("TkResolver++");
  String usr("psi");
  String pwd("isp");
  
  
  DEBUG_OUT << "Connecting to database..." << std::endl;
  std::auto_ptr<odbc::Connection> conn( odbc::DriverManager::getConnection(dsn, usr, pwd) );
  DEBUG_OUT << "...connected to database" << std::endl;
  
  DEBUG_OUT << "Getting metadata..." << std::endl;
  odbc::DatabaseMetaData *metadata( conn->getMetaData() );
  DEBUG_OUT << "...got metadata" << std::endl;
  
  
  DEBUG_OUT << "Getting columns..." << std::endl;
  std::auto_ptr<odbc::ResultSet> result( metadata->getColumns(String(""), String(""), String("experiment"), String("")) );
  DEBUG_OUT << "...got columns" << std::endl;
  
  
  DEBUG_OUT << "Iterating over column metadata..." << std::endl;
  for (result->next(); !result->isAfterLast(); result->next())
    {
      DEBUG_OUT << "  column: " << std::string(String(result->getString(4))) << std::endl;
    }
  DEBUG_OUT << "...iterated over column metadata" << std::endl;
}
